@@ -187,7 +187,8 @@ pango_win32_font_map_class_init (PangoFontMapClass *class)
   object_class->finalize = pango_win32_font_map_finalize;
   class->load_font = pango_win32_font_map_load_font;
   class->list_families = pango_win32_font_map_list_families;
-
+  class->shape_engine_type = PANGO_RENDER_TYPE_WIN32;
+  
   pango_win32_get_dc ();
 }
 
@@ -245,7 +246,7 @@ pango_win32_font_map_for_display (void)
   if (fontmap != NULL)
     return PANGO_FONT_MAP (fontmap);
 
-  fontmap = (PangoWin32FontMap *) g_type_create_instance (PANGO_TYPE_WIN32_FONT_MAP);
+  fontmap = g_object_new (PANGO_TYPE_WIN32_FONT_MAP, NULL);
   
   fontmap->font_cache = pango_win32_font_cache_new ();
   fontmap->freed_fonts = g_queue_new ();
