@@ -30,8 +30,12 @@
 /* For XExtSetCloseDisplay */
 #include <X11/Xlibint.h>
 
+#include "pango-engine-private.h"
 #include "pango-fontmap.h"
 #include "pango-utils.h"
+
+#undef PANGO_DISABLE_DEPRECATED
+
 #include "pangox-private.h"
 
 typedef struct _PangoXFamily       PangoXFamily;
@@ -216,6 +220,7 @@ pango_x_font_map_class_init (PangoFontMapClass *class)
   object_class->finalize = pango_x_font_map_finalize;
   class->load_font = pango_x_font_map_load_font;
   class->list_families = pango_x_font_map_list_families;
+  class->shape_engine_type = PANGO_RENDER_TYPE_X;
 }
 
 /*
@@ -541,7 +546,7 @@ pango_x_real_get_coverage_win (Display *display)
   int format;
   gulong n_items;
   gulong bytes_after;
-  Atom *data;
+  guchar *data;
   Window retval = None;
   int (*old_handler) (Display *, XErrorEvent *);
   
@@ -555,12 +560,12 @@ pango_x_real_get_coverage_win (Display *display)
 		      0, 4,
 		      False, XA_WINDOW,
 		      &type, &format, &n_items, &bytes_after,
-		      (guchar **)&data);
+		      &data);
   
   if (type == XA_WINDOW)
     {
       if (format == 32 && n_items == 1 && bytes_after == 0)
-	retval = *data;
+	retval = *(Atom *)data;
       
       XFree (data);
     }
@@ -573,11 +578,11 @@ pango_x_real_get_coverage_win (Display *display)
 			  0, 4,
 			  False, XA_WINDOW,
 			  &type, &format, &n_items, &bytes_after,
-			  (guchar **)&data) == Success &&
+			  &data) == Success &&
       type == XA_WINDOW)
     {
       if (format != 32 || n_items != 1 || bytes_after != 0 ||
-	  *data != retval)
+	  *(Atom *)data != retval)
 	retval = None;
       
       XFree (data);
@@ -1523,7 +1528,6 @@ pango_x_face_get_coverage (PangoXFace      *xface,
   PangoXFont *xfont;
   PangoXFontMap *xfontmap = NULL; /* Quiet gcc */
   PangoCoverage *result = NULL;
-  GHashTable *coverage_hash;
   Atom atom = None;
 
   if (xface)
@@ -1549,42 +1553,28 @@ pango_x_face_get_coverage (PangoXFace      *xface,
 
   if (!result)
     {
-      guint32 ch;
       PangoMap *shape_map;
-      PangoCoverage *coverage;
-      PangoCoverageLevel font_level;
-      PangoMapEntry *map_entry;
-      
+      PangoEngineShape *engine;
+      gunichar wc;
+
       result = pango_coverage_new ();
       
-      coverage_hash = g_hash_table_new (g_str_hash, g_str_equal);
-      
       shape_map = pango_x_get_shaper_map (language);
+      engine = (PangoEngineShape *)pango_map_get_engine (shape_map, PANGO_SCRIPT_COMMON);
       
-      for (ch = 0; ch < 65536; ch++)
+      for (wc = 0; wc < 65536; wc++)
 	{
-	  map_entry = pango_map_get_entry (shape_map, ch);
-	  if (map_entry->info)
-	    {
-	      coverage = g_hash_table_lookup (coverage_hash, map_entry->info->id);
-	      if (!coverage)
-		{
-		  PangoEngineShape *engine = (PangoEngineShape *)pango_map_get_engine (shape_map, ch);
-		  coverage = engine->get_coverage (font, language);
-		  g_hash_table_insert (coverage_hash, map_entry->info->id, coverage);
-		}
-	  
-	      font_level = pango_coverage_get (coverage, ch);
-	      if (font_level == PANGO_COVERAGE_EXACT && !map_entry->is_exact)
-		font_level = PANGO_COVERAGE_APPROXIMATE;
-	      
-	      if (font_level != PANGO_COVERAGE_NONE)
-		pango_coverage_set (result, ch, font_level);
-	    }
+	  PangoCoverageLevel level;
+
+	  level = _pango_engine_shape_covers (engine, font, language, wc);
+	  if (level != PANGO_COVERAGE_NONE)
+	    pango_coverage_set (result, wc, level);
 	}
       
-      g_hash_table_foreach (coverage_hash, free_coverages_foreach, NULL);
-      g_hash_table_destroy (coverage_hash);
+      return result;
+
+      
+      result = _pango_engine_shape_get_coverage (engine, font, language);
 
       if (atom)
 	pango_x_store_cached_coverage (xfontmap, atom, result);
